﻿Public Class frmTicTacToe

    Private Property Random As Random

    Private Property XImage As Bitmap

    Private Property CircleImage As Bitmap

    Public Property PlayerTurn As Boolean

    Public ReadOnly Property IsPlayerOnesTurn
        Get
            Return PlayerTurn
        End Get
    End Property

    Public ReadOnly Property IsPlayerTwosTurn
        Get
            Return Not PlayerTurn
        End Get
    End Property

    Sub New()
        InitializeComponent()
        Random = New Random()
        XImage = My.Resources.x
        CircleImage = My.Resources.circle
        InitializePictureBoxes()
    End Sub

    Private Sub InitializePictureBoxes()
        For i = 1 To 9
            Dim pb = CreatePictureBox()
            tlpGame.Controls.Add(pb)
        Next
    End Sub

    Private Function CreatePictureBox() As PictureBox
        Dim pb = New PictureBox()
        With pb
            .BackColor = Color.Transparent
            .Dock = DockStyle.Fill
            .Margin = New Padding(0)
            .BorderStyle = BorderStyle.FixedSingle
            AddHandler pb.Click, AddressOf PictureBox_Click
        End With
        Return pb
    End Function

    Private Sub PictureBox_Click(sender As Object, e As EventArgs)
        Dim pb = CType(sender, PictureBox)

        Dim tokenAlreadySet = pb.BackgroundImage IsNot Nothing
        If tokenAlreadySet Then
            Return
        End If

        SetPlayerToken(pb)

        If HasPlayerWon() Then
            AskForRematch()
        ElseIf AllTokensPlaced() Then
            AskForDrawRematch()
        Else
            ChangePlayerTurn()
        End If
    End Sub

    Private Sub SetPlayerToken(pb As PictureBox)
        If IsPlayerOnesTurn Then
            SetPlayerOneToken(pb)
        Else
            SetPlayerTwoToken(pb)
        End If
    End Sub

    Private Sub SetPlayerOneToken(pb As PictureBox)
        pb.BackgroundImage = XImage
    End Sub

    Private Sub SetPlayerTwoToken(pb As PictureBox)
        pb.BackgroundImage = CircleImage
    End Sub

    Private Function HasPlayerWon() As Boolean
        Dim horizontalMatch = CheckHorizontalMatch()
        If horizontalMatch Then
            Return True
        End If

        Dim verticalMatch = CheckVerticalMatch()
        If verticalMatch Then
            Return True
        End If

        Dim diagonalMatch = CheckDiagonalMatch()
        If diagonalMatch Then
            Return True
        End If

        Return False
    End Function

    Private Function CheckHorizontalMatch() As Boolean
        Return DoPictureBoxesMatch(1, 2, 3) OrElse
            DoPictureBoxesMatch(4, 5, 6) OrElse
            DoPictureBoxesMatch(7, 8, 9)
    End Function

    Private Function CheckVerticalMatch() As Boolean
        Return DoPictureBoxesMatch(1, 4, 7) OrElse
            DoPictureBoxesMatch(2, 5, 8) OrElse
            DoPictureBoxesMatch(3, 6, 9)
    End Function

    Private Function CheckDiagonalMatch() As Boolean
        Return DoPictureBoxesMatch(1, 5, 9) OrElse
            DoPictureBoxesMatch(7, 5, 3)
    End Function

    Private Function DoPictureBoxesMatch(ParamArray pictureBoxNumbers As Integer()) As Boolean
        Dim images = New List(Of PictureBox)
        For Each number In pictureBoxNumbers
            Dim pbIndex = number - 1
            Dim pb = CType(tlpGame.Controls(pbIndex), PictureBox)
            images.Add(pb)
        Next
        If IsPlayerOnesTurn Then
            Return images.All(Function(x) x.BackgroundImage Is XImage)
        Else
            Return images.All(Function(x) x.BackgroundImage Is CircleImage)
        End If
    End Function

    Private Sub AskForRematch()
        Dim playerNumber = IIf(IsPlayerOnesTurn, 1, 2)
        Dim msg = $"Player {playerNumber} won, do you want to restart?"
        Dim wantsToRestart = MessageBox.Show(msg, "Player won", MessageBoxButtons.YesNo, MessageBoxIcon.Question) = DialogResult.Yes
        If wantsToRestart Then
            Restart()
        Else
            Close()
        End If
    End Sub

    Private Sub AskForDrawRematch()
        Dim msg = $"It's a draw, do you want to restart?"
        Dim wantsToRestart = MessageBox.Show(msg, "Draw!", MessageBoxButtons.YesNo, MessageBoxIcon.Question) = DialogResult.Yes
        If wantsToRestart Then
            Restart()
        Else
            Close()
        End If
    End Sub

    Private Sub Restart()
        ResetImages()
        ChooseRandomStartingPlayer()
    End Sub

    Private Sub ResetImages()
        For Each pb In tlpGame.Controls.OfType(Of PictureBox)
            pb.BackgroundImage = Nothing
        Next
    End Sub

    Private Function AllTokensPlaced() As Boolean
        Dim countPlaced = 0
        For Each pb In tlpGame.Controls.OfType(Of PictureBox)
            Dim tokenSet = pb.BackgroundImage IsNot Nothing
            If tokenSet Then
                countPlaced += 1
            End If
        Next
        ' or with LINQ:
        ' Dim countPlaced = tlpGame.Controls.OfType(Of PictureBox).Where(Function(pb) pb.BackgroundImage IsNot Nothing)
        Return countPlaced = tlpGame.Controls.Count
    End Function

    Private Sub ChangePlayerTurn()
        PlayerTurn = Not PlayerTurn
        UpdatePlayerTurnLabel()
        StartCountDown()
    End Sub

    Private Sub UpdatePlayerTurnLabel()
        Dim playerNumber = 0
        If IsPlayerOnesTurn Then
            playerNumber = 1
        Else
            playerNumber = 2
        End If
        lblPlayerTurnValue.Text = $"Player {playerNumber}"
    End Sub

    Private Sub StartCountDown()
        lblTimeValue.Text = "10"
        tmrCountDown.Start()
    End Sub

    Private Sub frmTicTacToe_Load(sender As Object, e As EventArgs) Handles Me.Load
        ChooseRandomStartingPlayer()
    End Sub

    Private Sub ChooseRandomStartingPlayer()
        Dim rand = Random.Next(100)
        Dim isEven = rand Mod 2 = 0
        PlayerTurn = isEven
        UpdatePlayerTurnLabel()
        StartCountDown()
    End Sub

    Private Sub tmrCountDown_Tick(sender As Object, e As EventArgs) Handles tmrCountDown.Tick
        Dim time = Convert.ToByte(lblTimeValue.Text)
        If time > 0 Then
            time -= 1
            UpdateTimeLabel(time)
            Return
        End If
        tmrCountDown.Stop()
        ChangePlayerTurn()
        StartCountDown()
    End Sub

    Private Sub UpdateTimeLabel(time As Integer)
        lblTimeValue.Text = time.ToString()
    End Sub

    Private Sub frmTicTacToe_FormClosing(sender As Object, e As FormClosingEventArgs) Handles Me.FormClosing
        tmrCountDown.Stop()
    End Sub

    Private Sub btnBack_Click(sender As Object, e As EventArgs) Handles btnBack.Click
        Dim frmWelcome = New frmWelcome()
        frmWelcome.Show()
        Close()
    End Sub

    Private Sub btnExit_Click(sender As Object, e As EventArgs) Handles btnExit.Click
        Application.Exit()
    End Sub

End Class
